/*
 * Copyright (c) 2013-2017 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <signal.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include <rmp440/rmp440.h>

int done = 0;

void 
timespec_diff(struct timespec *start, struct timespec *stop,
                   struct timespec *result)
{
    if ((stop->tv_nsec - start->tv_nsec) < 0) {
        result->tv_sec = stop->tv_sec - start->tv_sec - 1;
        result->tv_nsec = stop->tv_nsec - start->tv_nsec + 1000000000;
    } else {
        result->tv_sec = stop->tv_sec - start->tv_sec;
        result->tv_nsec = stop->tv_nsec - start->tv_nsec;
    }

    return;
}

static void 
cc(int sig)
{
	done++;
}

int
main(int argc, char *argv[])
{
	struct rmp440_io *rmp;
	struct rmp440_feedback *fb;
	struct timespec tv1, tv2, duration;

	signal(SIGINT, cc);

	fb = rmp440FeedbackInit(NULL);

	rmp = rmp440Init(RMP440_INTERFACE_ETHERNET, "10.40.40.40:8080");
	if (rmp == NULL) {
		fprintf(stderr, "Cannot init rmp connection\n");
		exit(2);
	}	
#if 0
	rmp440SetUserFeedBack(rmp, 
	    0x00001000 |	/* The auxiliary battery state of charge */
	    
	    , 0);
#endif
	while (!done) {
		switch (fb->operational_state) {
		case 0:
		case 1:
		case 2:
		case 3:/* standby */
			rmp440CmdNone(rmp);
			break;
		case 4: /* tractor */
			rmp440SetOperationalMode(rmp, RMP_STANDBY_REQUEST);
			break;
		}
		clock_gettime(CLOCK_MONOTONIC, &tv1);
		if (rmp440ReceiveAndDecode(rmp, fb) < 0) {
			printf("Communication error with rmp440\n");
			done = 1;
		}
		clock_gettime(CLOCK_MONOTONIC, &tv2);
		timespec_diff(&tv1, &tv2, &duration);
		printf("duration: %lu:%09lu\n",
		    duration.tv_sec, duration.tv_nsec);
		printf("0x%08x 0x%08x 0x%08x 0x%08x\n",
		    fb->user_feedback_bitmap[0],
		    fb->user_feedback_bitmap[1],
		    fb->user_feedback_bitmap[2],
		    fb->user_feedback_bitmap[3]);
	}
	rmp440End(rmp);
	exit(0);
}
